--[[
	StrikeNow! A Power Monitor for Heroic Strike.

	This remains a work in progress. Originally a rage-dump monitor for Heroic Strike only, written mostly as a Warcraft API & LUA
	learning exercise.

	v2.00
		- Add resizing option for the monitor graphic
		- Disentangle threshold setting from glow in options  
		- Restructure and simplify glow code 
		- Add glow support for Bartender4

--]]

--[[
	To do:

	Possible Enhancements
		- Add full glow spell selection
		- Add other optional alerts (Sound/Text/Border flash)
		- Generalise to remove the "warrior-only" focus (would involve making glowed spells more selectable)
		- Make the Monitor Graphic clickable to cast?

	Known bugs
		- The Bartender4 support works, but isn't perfect, as the BT4 stance change code causes the buttons to be redrawn and
		  kills any active glows that this mod has added. This is a  problem if the StrikeNow! threshold value is not higher than
		  the power remaining after a stance change, as buttons that should be glowing won't be. Stance mastery and Tactical Mastery(2/2),
		  for example, could cause the problem for threshold values as high as 75. The code can restore deleted glows on the next power event
		  (likely to be the next tick) but it's still not ideal. Also, it's also probable that using some of the Bartender conditional
		  code (such as hiding a bar when in a vehicle) could cause a similar redraw, and whether or not the StrikeNow! code as it stands
		  would catch the "missing" glows is entirely down to what precise events would get driven.) 
		
--]]
	   
	SN_ADDON_NAME = "StrikeNow";
	SN = SN_ADDON_NAME.."!";

	SN_VERSION = '2.01';

	SN_DEBUG = false;
--	SN_DEBUG = true;


-- Locals

	local _G = _G;

-- Supported alternate bar managers
	local BT4;	-- Set to true if Bartender4 is installed

	

function StrikeNow_OnLoad(self)

	if ( SN_DEBUG )
	then
		DEFAULT_CHAT_FRAME:AddMessage("StrikeNow_OnLoad driven");
	end;

	local _, class = UnitClass("player");

	if ( class ~= "WARRIOR" )
	then  
		self:Hide();
		SN_ADD_TO_INTERFACE = false;

	else
		-- Initialise variables
		SN_ADD_TO_INTERFACE = true;
		SN_THRESHOLD_REACHED = false;

		self.spells = {};
		self.spells.watched = {};
		self.spells.found = {};
		self.spells.texture = {};
		-- self.spells.slots = {};

		self.addOnActive = true;
		self.talentGroups = 1;
		self.activeTalentGroup = 1;
		self.unitPowerPrev = 0;
		self.forceCheckCount = 0;

		StrikeNow_Data_BeltAndBraces();

		-- Register Slash commands
		SlashCmdList[SN_SLASH_STRING] = 	function(msg)
									StrikeNow_SlashCommandHandler(msg);
								end;


		-- Register initial events
		self:RegisterEvent("ADDON_LOADED");
		self:RegisterEvent("PLAYER_LOGIN");
		self:RegisterEvent("UNIT_POWER");

		self:RegisterEvent("ACTIVE_TALENT_GROUP_CHANGED");

		-- The next events are needed when glows are in use, to trigger appropriate rechecking.

		self:RegisterEvent("UPDATE_SHAPESHIFT_FORM");
		self:RegisterEvent("UPDATE_SHAPESHIFT_FORMS");

		self:RegisterEvent("ACTIONBAR_PAGE_CHANGED");
		self:RegisterEvent("ACTIONBAR_SLOT_CHANGED");

		-- Set the initial behaviour
		self:RegisterForDrag("LeftButton");
		self:EnableMouse(false);

		if ( DEFAULT_CHAT_FRAME )
		then
			local l_msg = SN .. " " .. SN_VERSION .. SN_LOAD_MSG;
			DEFAULT_CHAT_FRAME:AddMessage(l_msg);
		end;
	end;

end;


function StrikeNow_OnEvent(self, event, ... )

	if ( SN_DEBUG )
	then
		DEFAULT_CHAT_FRAME:AddMessage("StrikeNow_OnEvent driven");
	end;


	if (event == "ACTIVE_TALENT_GROUP_CHANGED")
	then
		StrikeNow_CheckTalentGroups(self, event, ... );
		StrikeNowOptions_SetData();

		if ( SN_ConfigData.useGlow )
		then
			StrikeNow_ForceThresholdCheck(self, event, ... );
		end;


	elseif (event == "ADDON_LOADED")
	then
		StrikeNow_AddonLoaded(self, event, ... );


	elseif ( event == "PLAYER_LOGIN" )
	then
		StrikeNow_OnLogin(self, event, ... );

		-- More event registration

		self:RegisterEvent("SPELLS_CHANGED");


		-- Check for supported button manager mods, and perform required initialisation
		-- according to the manager in use

		BT4 = Bartender4 and true or false;

		if ( BT4 )
		then
			if ( SN_DEBUG )
			then
				DEFAULT_CHAT_FRAME:AddMessage("Found Bartender4");

			end;
		else
			-- Register on a few events that the vanilla mod needs for glows
			self:RegisterEvent("ACTIVE_TALENT_GROUP_CHANGED");

			self:RegisterEvent("ACTIONBAR_PAGE_CHANGED");
			self:RegisterEvent("ACTIONBAR_SLOT_CHANGED");
		end;


	elseif (event == "SPELLS_CHANGED")
	then
		StrikeNow_CheckForSpells(self, event, ... );
		StrikeNowOptions_SetData();


	elseif ( event == "UNIT_POWER" )
	then

		local unitId = ...;

		if ( unitId == "player" )
		then
			StrikeNow_CheckThreshold(self, event, ... );
		end;


-- Events below this line are only used to force a threshold recheck when glows are enabled

	elseif (event == "ACTIONBAR_PAGE_CHANGED")
	then
		StrikeNow_ForceThresholdCheck(self, event, ... );


	elseif (event == "ACTIONBAR_SLOT_CHANGED")
	then
		StrikeNow_ForceThresholdCheck(self, event, ... );


	elseif (event == "UPDATE_SHAPESHIFT_FORM")
	then
		if ( BT4 )
		then
			self.forceCheckCount = 2;
		end;

		StrikeNow_ForceThresholdCheck(self, event, ... );


	elseif (event == "UPDATE_SHAPESHIFT_FORMS")
	then
		StrikeNow_ForceThresholdCheck(self, event, ... );

	end;
end;


function StrikeNow_OnLogin(self, event, ... )

	if ( SN_DEBUG )
	then
		DEFAULT_CHAT_FRAME:AddMessage("StrikeNow_OnLogin driven");
	end;

	StrikeNow_SetSpellList(self, event, ... );
	StrikeNow_CheckForSpells(self, event, ... );
	StrikeNow_CheckTalentGroups(self, event, ... );

end;


--
-- Set the list of spells to be managed by the mod
--
function StrikeNow_SetSpellList(self, event, ... )

	if ( SN_DEBUG )
	then
		DEFAULT_CHAT_FRAME:AddMessage("StrikeNow_SetSpellList driven");
	end;

	StrikeNow_AddSpellToList(self, SN_SPELL_HEROIC_STRIKE, true);

	if ( SN_ConfigData.useCleave ) 
	then
		StrikeNow_AddSpellToList(self, SN_SPELL_CLEAVE, true);
	end;

end;

--
-- Add a list entry for a single spell
--
function StrikeNow_AddSpellToList(self, newSpell, addSpell)

	if ( SN_DEBUG )
	then
		DEFAULT_CHAT_FRAME:AddMessage("StrikeNow_AddSpellToList driven");
	end;

	if ( addSpell )
	then
		self.spells.watched[newSpell] = true;
		self.spells.found[newSpell] = false;
	else
		self.spells.watched[newSpell] = nil;
		self.spells.found[newSpell] = nil;
	end;

	self.spells.texture[newSpell] = nil;

end; 


--
-- Locate the managed spells in the player's spell book
--
function StrikeNow_CheckForSpells(self, event, ... )

	if ( SN_DEBUG )
	then
		DEFAULT_CHAT_FRAME:AddMessage("StrikeNow_CheckForSpells driven");
	end;

	-- Reset the spell arrays
	self.spells.found = {};
	self.spells.texture = {};
	
	-- Find the spells

	local ss_spellName, ss_bool;

	for ss_spellName, ss_bool in pairs(self.spells.watched) do

		local i = 1;
		local skillType = GetSpellBookItemInfo(i,SpellBookFrame.bookType );
 		
		while ( skillType ~= nil and not self.spells.found[ss_spellName] ) do
		
			local spellName, spellRank = GetSpellInfo(i, SpellBookFrame.bookType)

	 		if ( skillType == "SPELL" and spellName == ss_spellName )
			then
				self.spells.found[ss_spellName] = true;
				self.spells.texture[ss_spellName] = GetSpellTexture(i,SpellBookFrame.bookType);

				if ( SN_DEBUG )
				then
					DEFAULT_CHAT_FRAME:AddMessage("Spell " .. ss_spellName .. " uses texture " .. self.spells.texture[ss_spellName]);
				end;

			end;
   
   			i = i + 1;
			skillType = GetSpellBookItemInfo(i,SpellBookFrame.bookType );
		end;
	end;

	if ( self.spells.found[SN_SPELL_HEROIC_STRIKE] )
	then
		StrikeNow_SetGraphicTexture(self)

		self:RegisterEvent("UNIT_POWER");
		StrikeNowOptions_Toggle_Show(true);
	else
		self:UnregisterEvent("UNIT_POWER");
		StrikeNowOptions_Toggle_Show(false);
	end;
				
end;


function StrikeNow_SetGraphicTexture(self)

		if ( SN_ConfigData.altMonitorGraphic )
		then
			_G[self:GetName().."_Graphic"]:SetTexture(SN_HS_ALT_TEXTURE);
		else
			_G[self:GetName().."_Graphic"]:SetTexture(self.spells.texture[SN_SPELL_HEROIC_STRIKE]);
		end;
end;


function StrikeNow_CheckTalentGroups(self, event, ... )

	if ( SN_DEBUG )
	then
		DEFAULT_CHAT_FRAME:AddMessage("StrikeNow_CheckTalentGroups driven");
	end;

	self.talentGroups = GetNumTalentGroups();
	self.activeTalentGroup = GetActiveTalentGroup(false, false );

	if ( self.talentGroups == 2 )
	then
		StrikeNowOptions_Show_Alt(true)
	else
		StrikeNowOptions_Show_Alt(false)
	end;

	if ( self.activeTalentGroup == 1 )
	then
		StrikeNowOptions_Main_Threshold_Active(true);
	else
		StrikeNowOptions_Main_Threshold_Active(false);
	end;

end;


function StrikeNow_CheckThreshold(self, event, ... )

	if ( SN_DEBUG )
	then
		DEFAULT_CHAT_FRAME:AddMessage("StrikeNow_CheckThreshold driven");
	end;


	local current_threshold;

	if ( self.activeTalentGroup == 1 )
	then
		current_threshold = SN_ConfigData.thresholdMain;
	else
		current_threshold = SN_ConfigData.thresholdAlt;
	end;

	local current_unitPower = UnitPower("player");

	if ( self.unitPowerPrev ~= current_unitPower or self.forceCheckCount > 0 )
	then

		if ( ( not SN_THRESHOLD_REACHED or self.forceCheckCount > 0 ) and current_unitPower >= current_threshold)
		then
			SN_THRESHOLD_REACHED = true;

			if ( SN_ConfigData.useGraphic ) 
			then
				_G[self:GetName().."_Graphic"]:Show();

			end;

			if ( SN_ConfigData.useGlow ) 
			then
				StrikeNow_UpdateAllGlows(self, true, ...);
			end;

		elseif ( ( SN_THRESHOLD_REACHED or self.forceCheckCount > 0 ) and current_unitPower < current_threshold)
		then
			SN_THRESHOLD_REACHED = false;

			if ( SN_ConfigData.useGraphic ) 
			then
				_G[self:GetName().."_Graphic"]:Hide();
			end;

			if ( SN_ConfigData.useGlow ) 
			then
				StrikeNow_UpdateAllGlows(self, false, ...);
			end;

		end;

		self.unitPowerPrev = current_unitPower;

		if ( self.forceCheckCount > 0 )
		then
			self.forceCheckCount = self.forceCheckCount - 1;
		end;
	end;
end;


--
-- Force a recheck of the threshold
--
function StrikeNow_ForceThresholdCheck(self, event, ... )

	if ( SN_DEBUG )
	then
		DEFAULT_CHAT_FRAME:AddMessage("StrikeNow_ForceThresholdCheck driven");
	end;

	if ( SN_THRESHOLD_REACHED )
	then
		SN_THRESHOLD_REACHED = false;
		self.unitPowerPrev = 0;
	else
		SN_THRESHOLD_REACHED = true;
		self.unitPowerPrev = 100;
	end;

	StrikeNow_CheckThreshold(self, event, ... );


end;


function StrikeNow_AddonLoaded(self, event, ... )

	if ( SN_DEBUG )
	then
		DEFAULT_CHAT_FRAME:AddMessage("StrikeNow_AddonLoaded driven");
	end;

	local addon = ...;

	if ( addon ~= SN_ADDON_NAME)
	then
		return;
	
	else
		StrikeNow_Init(self);
		StrikeNowOptions_SetData();

	end;
end;


function StrikeNow_Init(self)

	if ( SN_DEBUG )
	then
		DEFAULT_CHAT_FRAME:AddMessage("StrikeNow_Init driven");
	end;

	if ( not SN_ConfigData )
	then
		-- If no configuration was found, reset all data
		StrikeNow_Reset();
	
	else
		if ( SN_ConfigData.version ~= SN_VERSION )
		then
			StrikeNow_Migrate_Data();
		end;

		StrikeNow_Data_BeltAndBraces();
	end;

	self.initComplete = true;

end;


function StrikeNow_Reset()

	if ( SN_DEBUG )
	then
		DEFAULT_CHAT_FRAME:AddMessage("StrikeNow_Reset driven");
	end;

	-- Set the Monitor frame	
	_G[SN_STRIKENOW_ROOT_FRAME]:ClearAllPoints();
	_G[SN_STRIKENOW_ROOT_FRAME]:SetPoint("CENTER");
	StrikeNow_FrameUnlock(false);

	-- Set the Configuration data to its default values	
	SN_ConfigData = {};
	StrikeNow_Data_BeltAndBraces();

end;


function StrikeNow_FrameUnlock(checked)

	if ( SN_DEBUG )
	then
		DEFAULT_CHAT_FRAME:AddMessage("StrikeNow_FrameUnlock driven");
	end;

	_G[SN_STRIKENOW_ROOT_FRAME]:EnableMouse(checked);

	if ( checked )
	then
		_G[SN_STRIKENOW_ROOT_FRAME.."_Graphic"]:Show();
	else
		_G[SN_STRIKENOW_ROOT_FRAME.."_Graphic"]:Hide();
	end;

end;


function StrikeNow_GraphicSize()

	if ( SN_DEBUG )
	then
		DEFAULT_CHAT_FRAME:AddMessage("StrikeNow_GraphicResize driven");
	end;

	_G[SN_STRIKENOW_ROOT_FRAME]:SetSize(SN_ConfigData.graphicScale, SN_ConfigData.graphicScale);

end;


function StrikeNow_UseGraphic(checked)

	if ( SN_DEBUG )
	then
		DEFAULT_CHAT_FRAME:AddMessage("StrikeNow_UseGraphic driven");
	end;

	SN_ConfigData.useGraphic = checked;

end;


function StrikeNow_UseGlow(checked)

	if ( SN_DEBUG )
	then
		DEFAULT_CHAT_FRAME:AddMessage("StrikeNow_UseGlow driven");
	end;

	SN_ConfigData.useGlow = checked;

end;


function StrikeNow_UseCleave(checked)

	if ( SN_DEBUG )
	then
		DEFAULT_CHAT_FRAME:AddMessage("StrikeNow_UseCleave driven");
	end;

	SN_ConfigData.useCleave = checked;

end;



--
-- Code to Migrate any back-level SavedVariable content goes here
--
-- Previous versions explicitly handled: None
--
function StrikeNow_Migrate_Data()

	if ( SN_DEBUG )
	then
		DEFAULT_CHAT_FRAME:AddMessage("StrikeNow_Migrate_Data driven");
	end;

	-- Update the configuration version
	SN_ConfigData.version = SN_VERSION;

	-- Now do anything specific

	-- ((Nothing to do yet))

end;


--
-- Add any missing configuration variables
--
function StrikeNow_Data_BeltAndBraces()

	-- if ( SN_DEBUG )
	-- then
		DEFAULT_CHAT_FRAME:AddMessage("StrikeNow_Data_BeltAndBraces driven");
	-- end;

	if ( not SN_ConfigData )
	then
		SN_ConfigData = {};
	end;

	if ( not SN_ConfigData.version )
	then
		SN_ConfigData.version = SN_VERSION;
	end;

	if ( SN_DEBUG )
	then
		DEFAULT_CHAT_FRAME:AddMessage("StrikeNow_Data_BeltAndBraces driven");
	end;

	if ( not SN_ConfigData.useGraphic )
	then
		SN_ConfigData.useGraphic = false;
	end;

	if ( not SN_ConfigData.graphicScale )
	then
		SN_ConfigData.graphicScale = SN_GRAPHICSCALE_DEFAULT;
		DEFAULT_CHAT_FRAME:AddMessage("SN_ConfigData.graphicScale = "..SN_ConfigData.graphicScale);
	end;

	if ( not SN_ConfigData.altMonitorGraphic )
	then
		SN_ConfigData.altMonitorGraphic	= false;
	end;

	if ( not SN_ConfigData.useGlow )
	then
		SN_ConfigData.useGlow = false;
	end;

	if ( not SN_ConfigData.useCleave )
	then
		SN_ConfigData.useCleave = true;
	end;

	if ( not SN_ConfigData.thresholdMain )
	then
		SN_ConfigData.thresholdMain = SN_HS_THRESHOLD_DEFAULT;
	end;

	if ( not SN_ConfigData.thresholdAlt )
	then
		SN_ConfigData.thresholdAlt = SN_HS_THRESHOLD_DEFAULT;
	end;

end;


function StrikeNow_SlashCommandHandler(msg)

	if ( SN_DEBUG )
	then
		DEFAULT_CHAT_FRAME:AddMessage("StrikeNow_SlashCommandHandler driven");
	end;

	if ( msg )
	then
		local command = string.lower(msg);
		if( command == "" )
		then
			for index, value in next, SN_HELP_TEXT, nil do
				DEFAULT_CHAT_FRAME:AddMessage(value);
			end

		elseif( command == SN_UNLOCK)
		then
			StrikeNow_FrameUnlock(true);
			DEFAULT_CHAT_FRAME:AddMessage(SN_UNLOCK_DONE);

		elseif( command == SN_LOCK)
		then
			StrikeNow_FrameUnlock(false);
			DEFAULT_CHAT_FRAME:AddMessage(SN_LOCK_DONE);

		elseif( command == SN_RESET)
		then
			StrikeNow_Reset();
			StrikeNowOptions_SetData();
		end;
	end;

end;


--
-- Glow code
-- The map function called at (A) must return a table with an entry for each button (the object, not the name)
-- to which the spell slot passed to it is mapped by the bar manager in the current stance.
-- The table may be empty. The actual value associated with each entry in the table is not used.
--

function StrikeNow_UpdateAllGlows(self, startGlows, ...)

	if ( SN_DEBUG )
	then
		DEFAULT_CHAT_FRAME:AddMessage("StrikeNow_UpdateAllGlows driven");
	end;
	
	-- Get the stance
	local stance = GetShapeshiftForm(true); -- "true" returns 0 for Presences and Auras


	-- Process all the spell slots
	local slotNum;

	for slotNum=1,120 do

		-- Does slot "slotNum" contain a player spell?
		local slotType, slotId, slotSubType = GetActionInfo(slotNum);

		if ( slotType == "spell" and slotSubType == "spell" )
		then
			-- Is it one of the ones we're interested in?

			local slotSpellName = GetSpellInfo(slotId);
			local watchedSpellName, ss_bool;

			for watchedSpellName, ss_bool in pairs(self.spells.watched) do

				if ( slotSpellName == watchedSpellName ) 
				then
					local buttonSet; 
	


	-- (A) Call the appropriate mapping function to get the associated button names
					if ( BT4 )
					then
						buttonSet = StrikeNow_BT4_ButtonMap(self, slotNum, stance, ...);
					else
						buttonSet = StrikeNow_Blizzard_ButtonMap(self, slotNum, stance, ...);
					end;


					-- Change the glows on any buttons returned
					--  (note that the base UI code will check for button visibility, so we don't have to)

					local someVar;

					for SN_spellButton, someVar in pairs(buttonSet) do

						if ( startGlows )
						then
							ActionButton_ShowOverlayGlow(SN_spellButton);
						else
							ActionButton_HideOverlayGlow(SN_spellButton);
						end;
					end;
				end;

			end;

		end;

	end;

end;


--
-- Blizzard UI Warrior button map
--
-- 	Spell slots map in groups of 12 to logical action bars. Button names can vary according whether or not the
--	logical bar is currently being displayed as an additional bar, and in the case of logical bars being
--	displayed on the (scrollable) main action bar, whether or not the character is in a stance. 
--
function StrikeNow_Blizzard_ButtonMap(self, slotNum, stance, ...)

	if ( SN_DEBUG )
	then
		DEFAULT_CHAT_FRAME:AddMessage("StrikeNow_Blizzard_ButtonMap driven");
	end;

	local buttonSet = {};

	local buttonObject;
		
	-- Work out whether the button for this slot is on a visible bar, and if so, what
	-- the corresponding object is. Add any resulting object to the buttonSet table.
 
	local barNum = math.ceil(slotNum / 12);

	local buttonNameRoot = StrikeNow_Blizzard_ButtonNameRoot(stance, barNum, ...);

	if ( buttonNameRoot ) 
	then
		local barSpellNum = slotNum - ((barNum - 1) * 12 );
		buttonObject = _G[buttonNameRoot .. barSpellNum]

		if ( buttonObject )
		then
			buttonSet[buttonObject] = true;
		end;

	end;

	return buttonSet;

end;


--
--	Work out whether a given bar is currently visible, and, if so, return the string used as a prefix 
--	to form the names of buttons on the bar. 
--
--		If Action Bar {barNum} is currently visible,
--			the root string of the corresponding button names
--
--		If Action Bar {barNum} is NOT currently visible,
--			nil
--
--	Note:
--		It's technically possible, with the right combination of scrolling and option setting, to get
--		a single bar temporarily displayed both in the main position and as an additional bar. However,
--		that's arguably a bug in the base UI code, as the copy on the main bar is lost as soon as the
--		bar is scrolled. The spurious extra copy is therefore ignored by this function.
--		 
function StrikeNow_Blizzard_ButtonNameRoot(stance, barNum, ...)

	if ( SN_DEBUG )
	then
		DEFAULT_CHAT_FRAME:AddMessage("StrikeNow_ButtonNameRoot driven");
	end;

	-- Deliberate error to get line number
	-- SN_wibble( .. );

	local buttonNameRoot;

	local bottomLeftState, bottomRightState, sideRightState, sideRight2State = GetActionBarToggles();


	-- Check for the additional bars first
	if ( barNum == 3 and sideRightState )
	then
		buttonNameRoot = "MultiBarRightButton";

	elseif ( barNum == 4 and sideRight2State )
	then
		buttonNameRoot = "MultiBarLeftButton";

	elseif ( barNum == 5 and bottomRightState )
	then
		buttonNameRoot = "MultiBarBottomRightButton";

	elseif ( barNum == 6 and bottomLeftState )
	then
		buttonNameRoot = "MultiBarBottomLeftButton";

	else
	-- If visible, this bar can only be on the main action bar 

		local actionBarPage = GetActionBarPage();

		if ( stance == 0 )
		then
			-- Not in a stance; bar {barNum} is visible if it is the current scrolled action bar page
			if ( barNum == actionBarPage )
			then
				 buttonNameRoot = "ActionButton";
			end;
		else
			-- In a stance; A stance bar is visible if the barNum being tested is that for the stance AND the
			-- first action bar page is showing.
			-- A non-stance bar is visible if it is the current scrolled action bar page.

			if ( barNum == actionBarPage )
			then
				 buttonNameRoot = "BonusActionButton";
			else
				local stanceBarNum = stance + 6;

				if ( barNum == stanceBarNum and actionBarPage == 1 )
				then
					buttonNameRoot = "BonusActionButton";
				end;
			end;
		end;
	end;


	return buttonNameRoot;

end;


--
-- Bartender4 Warrior button map
--
-- 	As with the Blizzard UI, Spell slots map in groups of 12 to logical action bars. However, button naming
--	is rather simpler.
--
--	Each spell slot {n} maps directly to a button named "BT4Button{n}", with the following exception:
--   
--	 	If the character is in a stance, bar 1 is mapped to the same spell slots as bar 7, bar 8 or bar 9
--		(depending on the stance). Any or all of those bars may also be independently visible, exposing
--		a single spell slot as more than one button. Unlike the Blizzard UI, this is not a bug.
--
function StrikeNow_BT4_ButtonMap(self, slotNum, stance, ...)

	if ( SN_DEBUG )
	then
		DEFAULT_CHAT_FRAME:AddMessage("StrikeNow_BT4_ButtonMap driven");
	end;

	local buttonSet = {};
	local buttonNameString = "BT4Button";

	local buttonObject;

	if ( slotNum < 13 )
	then
		if ( stance == 0 )
		then
			buttonObject = _G[buttonNameString .. slotNum]
			if ( buttonObject )
			then
				buttonSet[buttonObject] = true;
			end;
		end;

	else
		buttonObject = _G[buttonNameString .. slotNum]
		if ( buttonObject )
		then
			buttonSet[buttonObject] = true;
		end;

		if ( slotNum > 72 )
		then
			if ( slotNum < 85 )
			then
				if ( stance == 1 )
				then
					buttonObject = _G[buttonNameString .. (slotNum - 72)]
					if ( buttonObject )
					then
						buttonSet[buttonObject] = true;
					end;
				end;

			elseif ( slotNum < 97 )
			then
				if ( stance == 2 )
				then
					buttonObject = _G[buttonNameString .. (slotNum - 84)]
					if ( buttonObject )
					then
						buttonSet[buttonObject] = true;
					end;
				end;

			elseif ( slotNum < 109 )
			then
				if ( stance == 3 )
				then
					buttonObject = _G[buttonNameString .. (slotNum - 96)]
					if ( buttonObject )
					then
						buttonSet[buttonObject] = true;
					end;
				end;
			end;
		end;
	end;

	return buttonSet;

end;